@@ -168,21 +168,23 @@ bool cFritzFonbuch::Initialize() {
 void cFritzFonbuch::Action() {
 	unsigned int retry_delay = RETRY_DELAY / 2;
 	std::string msg;
-	cFritzTools::getFritzBoxMutex()->Lock();
+	cFritzTools::GetFritzBoxMutex()->Lock();
 	do {
 		try {
 			retry_delay = retry_delay > 1800 ? 3600 : retry_delay * 2;
+			setInitialized(false);
 			fonbuchList.clear();
 			
-//			cFritzTools::getFritzBoxMutex()->Lock();
 			cFritzTools::Login();
 			dsyslog("fritzfonbuch.c: sending fonbuch request.");
 			cHttpClient tc(fritzboxConfig.url, PORT_WWW);
-			tc  <<	"GET /cgi-bin/webcm?getpage=../html/de/menus/menu2.html"
-					"&var:lang=de&var:pagename=fonbuch&var:menu=fon HTTP/1.1\n\n\0";
-
+			tc  <<	"GET /cgi-bin/webcm?getpage=../html/"
+				<<  cFritzTools::GetLang()
+				<<	"/menus/menu2.html"
+				<<	"&var:lang="
+				<<  cFritzTools::GetLang()
+				<<  "&var:pagename=fonbuch&var:menu=fon HTTP/1.1\n\n\0";
 			tc >> msg;
-			//cFritzTools::getFritzBoxMutex()->Unlock();
 			
 			// parse answer
 			size_t pos = 0, p1, p2;
@@ -242,44 +244,24 @@ void cFritzFonbuch::Action() {
 				pos = p2+10;
 				count++;
 			}
-			dsyslog("fritzfonbuch.c: read %i entries.", count);
+			isyslog("fritzfonbuch.c: read %i entries.", count);
 			setInitialized(true);
 		} catch (cTcpException te) {
 			//cFritzTools::getFritzBoxMutex()->Unlock();
-			dsyslog("fritzfonbuch.c: cTcpException - %s", te.what());
-			dsyslog("fritzfonbuch.c: waiting %u seconds before retrying", retry_delay);
+			esyslog("fritzfonbuch.c: cTcpException - %s", te.what());
+			esyslog("fritzfonbuch.c: waiting %u seconds before retrying", retry_delay);
 			sleep(retry_delay); // delay a possible retry
 		} catch (cToolsException te) {
 			//cFritzTools::getFritzBoxMutex()->Unlock();
-			dsyslog("fritzfonbuch.c: cToolsException - %s", te.what());
-			dsyslog("fritzfonbuch.c: waiting %u seconds before retrying", retry_delay);
+			esyslog("fritzfonbuch.c: cToolsException - %s", te.what());
+			esyslog("fritzfonbuch.c: waiting %u seconds before retrying", retry_delay);
 			sleep(retry_delay); // delay a possible retry
 		}
 	} while (!isInitialized());
-	cFritzTools::getFritzBoxMutex()->Unlock();
+	cFritzTools::GetFritzBoxMutex()->Unlock();
 	std::sort(fonbuchList.begin(), fonbuchList.end());
 }
 
 void cFritzFonbuch::Reload() {
-	setInitialized(false);
 	this->Start();
 }
-
-std::string cFritzFonbuch::ResolveToName(std::string &number) {
-	for (unsigned int pos=0; pos < fonbuchList.size(); pos++) {
-		//cFonbuchEntry fe = fonbuchList[pos];
-		if (cFritzTools::CompareNormalized(number, fonbuchList[pos].getNumber()) == 0)
-			return fonbuchList[pos].getName()+" "+fonbuchList[pos].getTypeName();
-	}
-	return number;
-}
-
-cFonbuchEntry *cFritzFonbuch::RetrieveFonbuchEntry(size_t id) {
-	if (id >= GetFonbuchSize())
-		return NULL;
-	return &fonbuchList[id];
-}
-
-size_t cFritzFonbuch::GetFonbuchSize() {
-	return fonbuchList.size();
-}
