@@ -7,20 +7,24 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- * 
+ *
  */
 
 #include "setup.h"
-#include "fonbuchmanager.h"
+#include "menu.h"
+#include <FonbookManager.h>
+#include <CallList.h>
+#include <Listener.h>
+#include <Config.h>
 #include <vdr/menuitems.h>
 
 #if VDRVERSNUM < 10509
@@ -29,62 +33,109 @@
 
 sFritzboxConfig fritzboxConfig;
 
-const char *cMenuSetupFritzbox::StoreMsn(){
+// set logging objects
+std::ostream *dlog = &std::clog;
+std::ostream *ilog = &std::cout;
+std::ostream *elog = &std::cerr;
+
+std::string cMenuSetupFritzbox::StoreMsn(){
 	std::vector<std::string>::iterator it;
-	std::string *msnLine = new std::string();
-	
-	
+	std::string msnLine;
+
 	for (it = fritzboxConfig.msn.begin(); it < fritzboxConfig.msn.end(); it++){
-		*msnLine += *it;
-		*msnLine += ' ';
+		msnLine += *it;
+		msnLine += ' ';
 	}
-	return msnLine->c_str(); // TODO: MemLeak?
+	return msnLine;
 }
 
-const char *cMenuSetupFritzbox::StoreFonbooks() {
+std::string cMenuSetupFritzbox::StoreFonbooks() {
 	std::vector<std::string>::iterator it;
-	std::string *fbLine = new std::string();
+	std::string fbLine;
 
 
 	for (it = fritzboxConfig.selectedFonbookIDs.begin(); it < fritzboxConfig.selectedFonbookIDs.end(); it++){
-		*fbLine += *it;
-		*fbLine += ' ';
-	}		
-	return fbLine->c_str(); // TODO: MemLeak?
+		fbLine += *it;
+		fbLine += ' ';
+	}
+	return fbLine;
 }
 
 void cMenuSetupFritzbox::Setup(void) {
-	// save current postion
+	// save current position
 	int current = Current();
 	// clear entries, if any
-		Clear();
+	Clear();
+	int ret;
+	//possible values for "React on calls"
+	ret = asprintf(&directions[fritzboxConfig.DIRECTION_ANY], tr("any"));
+	if (ret <= 0) {
+		ERR("Error allocating linebuffer for cOsdItem.");
+		return;
+	}
+	ret = asprintf(&directions[fritzboxConfig.DIRECTION_IN],  tr("incoming"));
+	if (ret <= 0) {
+		ERR("Error allocating linebuffer for cOsdItem.");
+		return;
+	}
+	ret = asprintf(&directions[fritzboxConfig.DIRECTION_OUT], tr("outgoing"));
+	if (ret <= 0) {
+		ERR("Error allocating linebuffer for cOsdItem.");
+		return;
+	}
+	//possible values for "Default Menu"
+	ret = asprintf(&menus[cMenuFritzbox::FONBUCH], tr("PB"));
+	if (ret <= 0) {
+		ERR("Error allocating linebuffer for cOsdItem.");
+		return;
+	}
+	ret = asprintf(&menus[cMenuFritzbox::IN], tr("incoming"));
+	if (ret <= 0) {
+		ERR("Error allocating linebuffer for cOsdItem.");
+		return;
+	}
+	ret = asprintf(&menus[cMenuFritzbox::OUT], tr("outgoing"));
+	if (ret <= 0) {
+		ERR("Error allocating linebuffer for cOsdItem.");
+		return;
+	}
+	ret = asprintf(&menus[cMenuFritzbox::MISSED], tr("missed"));
+	if (ret <= 0) {
+		ERR("Error allocating linebuffer for cOsdItem.");
+		return;
+	}
 	// build up setup menu
 	Add(new cMenuEditStrItem (tr("Fritz!Box URL"),                  		url,           			MaxFileName, tr(FileNameChars)));
 	Add(new cMenuEditStrItem (tr("Password"),                       		password, 	   			MaxFileName, tr(FileNameChars)));
-//	Add(new cMenuEditStrItem (tr("Country code"),                           countryCode,            6,           "0123456789+"    ));
-	Add(new cMenuEditBoolItem(tr("Mute on call"),                   		&muteOnCall,   			trVDR("no"), 	 trVDR("yes")));
-	Add(new cMenuEditBoolItem(tr("Pause on call"),                   		&pauseOnCall,  			trVDR("no"), 	 trVDR("yes")));
-	Add(new cMenuEditBoolItem(tr("Show incoming calls"),            		&showNumber,   			trVDR("no"), 	 trVDR("yes")));
-	Add(new cMenuEditBoolItem(tr("Detailed call lists"),                    &showNumberInCallList, 	trVDR("no"), 	 trVDR("yes")));
-	Add(new cMenuEditBoolItem(tr("Group call lists by date"),               &showDaySeparator,      trVDR("no"),     trVDR("yes")));
-	Add(new cMenuEditBoolItem(tr("Hide main menu entry"), 		    		&hideMainMenu, 			trVDR("no"),  	 trVDR("yes")));
-	Add(new cOsdItem         (tr("Setup phonebooks to use..."), osUser1                                                          ));
-	Add(new cMenuEditBoolItem(tr("Restrict monitor to certain extensions"), &msnFilter,    			trVDR("no"), 	 trVDR("yes")));
+	Add(new cMenuEditStrItem (tr("Country code"),                           countryCode,            5,           "0123456789"));
+	Add(new cMenuEditStrItem (tr("Region code"),                            regionCode,             10,          "0123456789"));
+	Add(new cMenuEditStraItem(tr("React on calls"),                         &reactOnDirection,      3,           directions  ));
+	Add(new cMenuEditBoolItem(tr("Mute on call"),                   		&muteOnCall,   			trVDR("no"), trVDR("yes")));
+	Add(new cMenuEditBoolItem(tr("Pause on call"),                   		&pauseOnCall,  			trVDR("no"), trVDR("yes")));
+	Add(new cMenuEditBoolItem(tr("Show calls"),            		            &showNumber,   			trVDR("no"), trVDR("yes")));
+	Add(new cMenuEditBoolItem(tr("Show detailed call information"),         &useNotifyOsd,          trVDR("no"), trVDR("yes")));
+	Add(new cMenuEditBoolItem(tr("Detailed call lists"),                    &showNumberInCallList, 	trVDR("no"), trVDR("yes")));
+	Add(new cMenuEditBoolItem(tr("Group call lists by date"),               &showDaySeparator,      trVDR("no"), trVDR("yes")));
+	Add(new cMenuEditBoolItem(tr("Hide main menu entry"), 		    		&hideMainMenu, 			trVDR("no"), trVDR("yes")));
+	Add(new cMenuEditStraItem(tr("Default menu"),                           &defaultMenu,           4,           menus       ));
+	Add(new cOsdItem         (tr("Setup phonebooks to use..."),             osUser1                                          ));
+	Add(new cMenuEditBoolItem(tr("Restrict monitor to certain extensions"), &msnFilter,    			trVDR("no"), trVDR("yes")));
 	if (msnFilter) {
 		Add(new cMenuEditIntItem (tr("Number of monitored extensions"), &msnCount, 1, MAX_MSN_COUNT));
 		for (int p = 0; p < msnCount; p++)
-			Add(new cMenuEditStrItem(tr("Extension"), msn[p], MaxFileName, "1234567890"));	
-	}	
+			Add(new cMenuEditStrItem(tr("Extension"), msn[p], MaxFileName, "1234567890"));
+	}
+	SetHelp(tr("Reload"));
 	// restore current position
 	SetCurrent(Get(current));
 	// refresh display
 	Display();
-}	
+}
 
 eOSState cMenuSetupFritzbox::ProcessKey(eKeys Key) {
 
 	eOSState state = cMenuSetupPage::ProcessKey(Key);
-	
+
 	if (state == osUser1) {
 		return AddSubMenu(new cMenuSetupFritzboxFonbooks(&selectedFonbookIDs));
 	}
@@ -106,9 +157,9 @@ eOSState cMenuSetupFritzbox::ProcessKey(eKeys Key) {
 					msn[i] = (char *) malloc(MaxFileName * sizeof(char));
 					msn[i][0] = 0;
 				}
-			} 
-			// remove existing MSN elements				
-			else {				
+			}
+			// remove existing MSN elements
+			else {
 				for (int i=msnCountBefore; i>msnCount; i--)
 					free(msn[i-1]);
 			}
@@ -116,6 +167,18 @@ eOSState cMenuSetupFritzbox::ProcessKey(eKeys Key) {
 			msnCountBefore = msnCount;
 		}
 	}
+
+	if (state == osUnknown) {
+		switch (Key) {
+		case kRed:
+			fritz::FonbookManager::GetFonbook()->Reload();
+			Skins.QueueMessage(mtInfo, tr("Retrieving phone book"));
+			state = osContinue;
+			break;
+		default:
+			break;
+		}
+	}
 	return state;
 }
 
@@ -125,53 +188,60 @@ void cMenuSetupFritzbox::Store(void) {
 	// only store the password if it was changed
 	while (password[i]) {
 		if (password[i] != '*') {
-			fritzboxConfig.password       		= password;		
+			fritzboxConfig.password 	= password;
 			break;
 		}
 		i++;
 	}
-//	fritzboxConfig.countryCode          = countryCode;
-//	if (countryCode[0] == '0' && countryCode[1] == '0')
-//		fritzboxConfig.countryCode.replace(0, 2, "+");
+	// accept empty password
+	if (password[0] == 0)
+		fritzboxConfig.password = "";
+	//
+	fritzboxConfig.reactOnDirection     = reactOnDirection;
 	fritzboxConfig.muteOnCall     		= muteOnCall;
 	fritzboxConfig.pauseOnCall     		= pauseOnCall;
 	fritzboxConfig.showNumber     		= showNumber;
+	fritzboxConfig.useNotifyOsd         = useNotifyOsd;
 	fritzboxConfig.showNumberInCallList = showNumberInCallList;
 	fritzboxConfig.showDaySeparator     = showDaySeparator;
 	fritzboxConfig.hideMainMenu   		= hideMainMenu;
+	fritzboxConfig.defaultMenu          = defaultMenu;
 	fritzboxConfig.msn.clear();
 	for (int i=0; i < msnCount; i++) {
 		std::string s = msn[i];
 		fritzboxConfig.msn.push_back(s);
 	}
-	for (size_t i=0; i<selectedFonbookIDs.size(); i++) {
-		bool initialize = true;
-		for (size_t j=0; j<fritzboxConfig.selectedFonbookIDs.size(); j++) {
-			if (selectedFonbookIDs[i].compare(fritzboxConfig.selectedFonbookIDs[j]) == 0) {
-				initialize = false;
-				break;
-			}
-		}
-		if (initialize)
-			(*(cFonbuchManager::GetFonbuchManager()->GetFonbooks()))[selectedFonbookIDs[i]]->Initialize();
-	}
 	fritzboxConfig.selectedFonbookIDs   = selectedFonbookIDs;
-	
+    fritzboxConfig.countryCode          = countryCode;
+    fritzboxConfig.regionCode           = regionCode;
+	// remove any leading zeros from countryCode and regionCode,
+	while (!fritzboxConfig.countryCode.empty() && fritzboxConfig.countryCode[0] == '0')
+		fritzboxConfig.countryCode = fritzboxConfig.countryCode.substr(1);
+	while (!fritzboxConfig.regionCode.empty() && fritzboxConfig.regionCode[0] == '0')
+		fritzboxConfig.regionCode = fritzboxConfig.regionCode.substr(1);
+
+	((cThread *)fritzbox)->Start(); // re-read configuration, notify libfritz++ about changes
+
 	SetupStore("Url",          			url);
 	SetupStore("Password",     			fritzboxConfig.password.c_str());
-//	SetupStore("CountryCode",           fritzboxConfig.countryCode.c_str());
+	SetupStore("ReactOnDirection",      reactOnDirection);
 	SetupStore("MuteOnCall",   			muteOnCall);
 	SetupStore("PauseOnCall",  			pauseOnCall);
 	SetupStore("ShowNumber",   			showNumber);
+	SetupStore("UseNotifyOsd",          useNotifyOsd);
 	SetupStore("ShowNumberInCallList", 	showNumberInCallList);
 	SetupStore("ShowDaySeparator",      showDaySeparator);
 	SetupStore("HideMainMenu", 			hideMainMenu);
-	SetupStore("MsnList",      			StoreMsn());
-	SetupStore("Fonbooks",              StoreFonbooks());
+	SetupStore("DefaultMenu",           defaultMenu);
+	SetupStore("MsnList",      			StoreMsn().c_str());
+	SetupStore("Fonbooks",              StoreFonbooks().c_str());
+	SetupStore("CountryCode",           fritzboxConfig.countryCode.c_str());
+	SetupStore("RegionCode",            fritzboxConfig.regionCode.c_str());
 }
 
-cMenuSetupFritzbox::cMenuSetupFritzbox()	
+cMenuSetupFritzbox::cMenuSetupFritzbox(cPluginFritzbox *fritzbox)
 {
+	this->fritzbox = fritzbox;
 	// copy setup to temporary parameters
 	msn = (char **) malloc(MAX_MSN_COUNT * sizeof(char *));
 	url             	 = strdup(fritzboxConfig.url.c_str());
@@ -181,19 +251,23 @@ cMenuSetupFritzbox::cMenuSetupFritzbox()
 	for (unsigned int i = 0; i < fritzboxConfig.password.length(); i++) {
 		password[i] = '*';
 	}
-//	countryCode          = strdup(fritzboxConfig.countryCode.c_str());
+	reactOnDirection     = fritzboxConfig.reactOnDirection;
 	muteOnCall      	 = fritzboxConfig.muteOnCall;
 	pauseOnCall      	 = fritzboxConfig.pauseOnCall;
 	showNumber      	 = fritzboxConfig.showNumber;
+	useNotifyOsd         = fritzboxConfig.useNotifyOsd;
 	showNumberInCallList = fritzboxConfig.showNumberInCallList;
 	showDaySeparator     = fritzboxConfig.showDaySeparator;
 	hideMainMenu    	 = fritzboxConfig.hideMainMenu;
+	defaultMenu			 = fritzboxConfig.defaultMenu;
 	msnCount        	 = fritzboxConfig.msn.size();
 	msnCountBefore  	 = msnCount; // needed for menu refresh
 	msnFilter       	 = fritzboxConfig.msn.empty() ? 0 : 1;
 	msnFilterBefore 	 = msnFilter;
 	selectedFonbookIDs   = fritzboxConfig.selectedFonbookIDs;
-	
+	countryCode          = strdup(fritzboxConfig.countryCode.c_str());
+	regionCode           = strdup(fritzboxConfig.regionCode.c_str());
+
 	size_t p = 0;
 	for(std::vector<std::string>::iterator itStr = fritzboxConfig.msn.begin(); itStr < fritzboxConfig.msn.end(); itStr++) {
 		msn[p] = (char *) malloc(MaxFileName * sizeof(char));
@@ -209,23 +283,31 @@ cMenuSetupFritzbox::~cMenuSetupFritzbox()
 	// free up malloced space from constructor
 	free(url);
 	free(password);
-//	free(countryCode);
 	for (int i=0; i<msnCount; i++)
 		free(msn[i]);
 	free(msn);
+	for (int i=0; i<3; i++)
+		free(directions[i]);
+	for (int i=0; i<4; i++)
+		free(menus[i]);
+	free(countryCode);
+	free(regionCode);
 }
 
 cMenuSetupFritzboxFonbooks::cMenuSetupFritzboxFonbooks(std::vector<std::string> *selectedFonbookIDs)
 :cOsdMenu(tr("Setup phonebooks to use"), 4)
 {
-	fonbooks = cFonbuchManager::GetFonbuchManager()->GetFonbooks();
+	fonbooks = fritz::FonbookManager::GetFonbookManager()->GetFonbooks();
 	this->selectedFonbookIDs = selectedFonbookIDs;
 	// copy setup to temporary parameters
 	numberOfSelectedFonbooks = selectedFonbookIDs->size();
 	selectedFonbookPos = (int **)  malloc(fonbooks->size() * sizeof(int *));
 	fonbookTitles      = (char **) malloc(fonbooks->size() * sizeof(char *));
-	for (size_t i=0; i<fonbooks->size(); i++) {			
-		asprintf(&fonbookTitles[i], "%s", (*fonbooks)[i]->GetTitle().c_str());
+	for (size_t i=0; i<fonbooks->size(); i++) {
+		int ret = asprintf(&fonbookTitles[i], "%s", tr((*fonbooks)[i]->GetTitle().c_str()));
+		if (ret <= 0) {
+			ERR("Error allocating linebuffer for cOsdItem.");
+		}
 		selectedFonbookPos[i] = (int *) malloc(sizeof(int));
 	}
 	// build up menu entries
@@ -249,15 +331,19 @@ void cMenuSetupFritzboxFonbooks::Setup(void) {
 	// save current postion
 	int current = Current();
 	// clear entries, if any
-		Clear();
+	Clear();
 	// build up setup menu
 	for (size_t i=0; i<numberOfSelectedFonbooks; i++) {
 		char *numberStr;
-		asprintf(&numberStr, "%i.", i+1);
+		int ret = asprintf(&numberStr, "%i.", (int) (i+1));
+		if (ret <= 0) {
+			ERR("Error allocating linebuffer for cOsdItem.");
+				return;
+			}
 		size_t pos = 0;
 		if (i < selectedFonbookIDs->size())
-			while (pos < fbCount && 
-				  (*fonbooks)[pos]->GetTechId().compare((*selectedFonbookIDs)[i]) != 0) 
+			while (pos < fbCount &&
+					(*fonbooks)[pos]->GetTechId().compare((*selectedFonbookIDs)[i]) != 0)
 				pos++;
 		*(selectedFonbookPos[i]) = (int) pos;
 		Add(new cMenuEditStraItem(numberStr, selectedFonbookPos[i], fbCount, fonbookTitles));
@@ -291,7 +377,7 @@ eOSState cMenuSetupFritzboxFonbooks::ProcessKey(eKeys Key) {
 		case kOk:
 			selectedFonbookIDs->clear();
 			for (size_t i=0; i<numberOfSelectedFonbooks; i++) {
-				std::string s = (*fonbooks)[*selectedFonbookPos[i]]->GetTechId();	
+				std::string s = (*fonbooks)[*selectedFonbookPos[i]]->GetTechId();
 				selectedFonbookIDs->push_back(s);
 			}
 			state = osBack;
@@ -303,18 +389,25 @@ eOSState cMenuSetupFritzboxFonbooks::ProcessKey(eKeys Key) {
 }
 
 sFritzboxConfig::sFritzboxConfig() {
+	configDir               = "";
+	pluginName              = "";
+	lang                    = "";
 	url             		= "fritz.box";
-	password        		= "your password";
-//	countryCode             = "+49";
-	countryCode             = "";
+	password        		= "";
+	countryCode             = "49";
 	regionCode              = "";
+	reactOnDirection        = DIRECTION_IN;
 	muteOnCall      		= 0;
 	pauseOnCall      		= 0;
 	showNumber      		= 1;
+	useNotifyOsd            = 0;
 	showNumberInCallList    = 0;
+	lastKnownMissedCall     = 0;
 	showDaySeparator        = 1;
 	hideMainMenu    		= 0;
+	defaultMenu             = cMenuFritzbox::FONBUCH;
 	selectedFonbookIDs.push_back("FRITZ");
+	activeFonbookID         = "FRITZ";
 }
 
 bool sFritzboxConfig::SetupParseMsn(const char *value){
@@ -322,7 +415,7 @@ bool sFritzboxConfig::SetupParseMsn(const char *value){
 	unsigned int pos = 0;
 	// walk through the complete value-line
 	while (value[pos] != 0) {
-		currentMsn.erase(); 
+		currentMsn.erase();
 		// stop at each <space> or EOL
 		while (value[pos] != ' ' && value[pos] != 0) {
 			currentMsn += value[pos];
@@ -342,7 +435,7 @@ bool sFritzboxConfig::SetupParseFonbooks(const char *value){
 	selectedFonbookIDs.clear();
 	// walk through the complete value-line
 	while (value[pos] != 0) {
-		currentFb.erase(); 
+		currentFb.erase();
 		// stop at each <space> or EOL
 		while (value[pos] != ' ' && value[pos] != 0) {
 			currentFb += value[pos];
@@ -359,15 +452,21 @@ bool sFritzboxConfig::SetupParseFonbooks(const char *value){
 bool sFritzboxConfig::SetupParse(const char *name, const char *value) {
 	if      (!strcasecmp(name, "Url"))          		url          		 = value;
 	else if (!strcasecmp(name, "Password"))     		password     		 = value;
-//	else if (!strcasecmp(name, "CountryCode"))     		countryCode   		 = value;
+	else if (!strcasecmp(name, "ReactOnDirection"))     reactOnDirection     = atoi(value);
 	else if (!strcasecmp(name, "MuteOnCall"))   		muteOnCall   		 = atoi(value);
 	else if (!strcasecmp(name, "PauseOnCall"))   		pauseOnCall   		 = atoi(value);
 	else if (!strcasecmp(name, "ShowNumber"))   		showNumber   		 = atoi(value);
+	else if (!strcasecmp(name, "UseNotifyOsd"))         useNotifyOsd         = atoi(value);
 	else if (!strcasecmp(name, "ShowNumberInCallList")) showNumberInCallList = atoi(value);
+	else if (!strcasecmp(name, "LastKnownMissedCall"))  lastKnownMissedCall  = atoi(value);
 	else if (!strcasecmp(name, "ShowDaySeparator"))     showDaySeparator     = atoi(value);
 	else if (!strcasecmp(name, "HideMainMenu")) 		hideMainMenu 		 = atoi(value);
+	else if (!strcasecmp(name, "DefaultMenu"))          defaultMenu          = atoi(value);
+	else if (!strcasecmp(name, "ActiveFonbook"))        activeFonbookID      = value;
 	else if (!strcasecmp(name, "MsnList"))      		return SetupParseMsn(value);
 	else if (!strcasecmp(name, "Fonbooks"))      		return SetupParseFonbooks(value);
+	else if (!strcasecmp(name, "CountryCode"))          countryCode          = value;
+	else if (!strcasecmp(name, "RegionCode"))           regionCode           = value;
 	else return false;
 	return true;
 }
